package app.wordexam;

import app.wordexam.model.ElementInfo;
import app.wordexam.model.Item;
import app.wordexam.model.QuestionInfo;
import app.wordexam.service.ElementService;
import app.wordexam.service.FormService;
import app.wordexam.service.QuestionService;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;
import org.apache.commons.io.FileUtils;
import org.apache.commons.io.FilenameUtils;
import org.jsoup.Jsoup;
import org.jsoup.nodes.Document;
import org.jsoup.nodes.Element;
import org.jsoup.select.Elements;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.CommandLineRunner;
import org.springframework.boot.SpringApplication;
import org.springframework.boot.autoconfigure.SpringBootApplication;

import java.io.BufferedWriter;
import java.io.File;
import java.io.IOException;
import java.nio.charset.StandardCharsets;
import java.nio.file.Files;
import java.nio.file.Path;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.concurrent.atomic.AtomicBoolean;
import java.util.concurrent.atomic.AtomicReference;
import java.util.stream.Collectors;

@SpringBootApplication
public class WordexamApplication implements CommandLineRunner {

    @Autowired
    ElementService elementService;

    @Autowired
    QuestionService questionService;

    @Autowired
    FormService formService;


    File output = new File("output");
    File doc = new File("doc");

    public static void main(String[] args) {
        SpringApplication.run(WordexamApplication.class, args);
    }


    public Map<Integer, List<Element>> getQuestions(List<ElementInfo> elementInfos) {
        int index = 0;
        List<Element> question = new ArrayList<>();
        Map<Integer, List<Element>> questions = new HashMap<>();
        for (int i = 0; i < elementInfos.size(); i++) {
            if (index != elementInfos.get(i).getIndex()) {
                questions.put(index, question);
                index = elementInfos.get(i).getIndex();
                question = new ArrayList<>();
            }
            question.add(elementInfos.get(i).getElement());

            if (i == (elementInfos.size() - 1)) {
                questions.put(index, question);
            }

        }

        return questions;
    }


    @Override
    public void run(String... args) throws Exception {

        ObjectMapper mapper = new ObjectMapper();
        //mkdir outputdir and clean
        if (!output.exists()) {
            output.mkdir();
        }


        //get html files
        List<Path> htmlfiles = new ArrayList<>();
        htmlfiles = Files.list(Path.of(doc.toURI()))
                .filter(file -> file.toString().endsWith("html") || file.toString().endsWith("htm"))
                .collect(Collectors.toList());

        if (htmlfiles.size() == 0 || htmlfiles == null) {
            System.out.println("no html file found");
            System.exit(0);
        }

        for (int i = 0; i < htmlfiles.size(); i++) {
            Path htmlfile = htmlfiles.get(i);

            String mainTitle = FilenameUtils.removeExtension(htmlfile.getFileName().toString());
            //mkdir
            new File(String.format("%s/%s", output.toString(), mainTitle)).mkdir();
            Document document = Jsoup.parse(new File(htmlfile.toString()), "UTF-8");

            Elements elements = document.select("p");

            List<ElementInfo> elementInfos = new ArrayList<>();
            elementInfos = elementService.getElementInfo(elements);
            Map<Integer, List<Element>> questions = new HashMap<>();

            questions = getQuestions(elementInfos);
            AtomicReference<String> imgFolder = new AtomicReference<>("");
            List<QuestionInfo> questionInfoList = new ArrayList<>();
            AtomicBoolean hasImgFolder = new AtomicBoolean(false);
            Map<String, String> imagesCollection = new HashMap<>();

            questions.forEach((index, question) -> {

                //get imgFolder
                if (imgFolder.get().length() == 0) {
                    imgFolder.set(questionService.getImgFolder(question));

                }

                //get ans
                Map<String, List<Item>> ans = new HashMap<>();
                ans = questionService.getAns(question);
                QuestionInfo questionInfo = new QuestionInfo(index, ans);

                //get title
//                questionService.getTitle(question);
                Map<String, List<Item>> title = new HashMap<>();
                    title = questionService.getTitle(question);
//                    System.out.println(mapper.writeValueAsString(title));
                questionInfo.getInfomation().put("title", title.get("title"));

                //get choices
                Map<String, List<Item>> choices = new HashMap<>();
                choices = questionService.getChoices(question);

                if (choices.size() == 0) {
                    System.out.println("no choices, exit");
                    System.exit(0);
                }

                questionInfo.getInfomation().put("choices", choices.get("choices"));
                questionInfoList.add(questionInfo);


                //get images
                //取得每個image的base64
                Map<String, String> images = formService.getImages(questionInfo);

                if (images.size() > 0) {
                    imagesCollection.putAll(images);
                }
                images.forEach((image, encode) -> {
                    File srcFile = new File(String.format("doc/%s", image));
                    File dstFile = new File(String.format("%s/%s/%s", output.toString(), new File(mainTitle), image));

                    try {
                        FileUtils.copyFile(srcFile, dstFile);
                        hasImgFolder.set(true);
                    } catch (IOException e) {
                        e.printStackTrace();
                    }


                });


            });


            //開始寫出script
            System.out.println("檔案: " + mainTitle + ",共" + questionInfoList.size() + "題");
            String questionmodels = formService.getQuestionModel(questionInfoList);
//        System.out.println(questionmodels);
            //write to script

            String scriptfile = String.format("%s/%s/%s轉換批次檔.txt", output.toString(), mainTitle, mainTitle);

            BufferedWriter writer = Files.newBufferedWriter(Path.of(scriptfile), StandardCharsets.UTF_8);

            Calendar calendar = Calendar.getInstance();
            SimpleDateFormat timestamp = new SimpleDateFormat("yyyyMMddHHmmss");

            //執行的functionn名稱, 以時間戳記產生
            writer.append(String.format("function main%s() {", timestamp.format(calendar.getTime())));
            writer.append("\n");
            writer.append("  let scores = 10;  //每題配分  \n" +
                    "  let isAllRequired = true; //是否每題都要作答 true, false\n");

            writer.append("\n\n");
            writer.append("////////////////以下不用編輯//////////////\n");
            writer.append(String.format("  let form = FormApp.create(\"%s\")", mainTitle));
            writer.append("\n");
            writer.append(String.format("  form.setTitle(\"%s\")", mainTitle));
            writer.append("\n");
            writer.append("  " +
                    "  form.setIsQuiz(true);\n" +
                    "\n" +
                    "  let userinfo = form.addTextItem();\n" +
                    "  userinfo.setTitle('請輸入您的班級座號,例如：6101');\n" +
                    "  userinfo.setRequired(true);");

            writer.append("\n");
            writer.append("//開始處理題目\n");
            writer.append("\n");

            writer.append("//圖片資料庫\n");
            writer.append(String.format("imagesDB=%s;", mapper.writeValueAsString(imagesCollection)));
            writer.append("\n");
            writer.append("\n");



            writer.append(" \n");
            writer.append(" questions=" + questionmodels);
            writer.append(" \n");
            writer.append(" \n");

            writer.append("questions.forEach(question => {\n" +
                    "\n" +
                    "\n" +
                    "    let titleImgs = [];\n" +
                    "    let titleTxts = [];\n" +
                    "    question.title.forEach((item) => {\n" +
                    "\n" +
                    "      //取出題目說明\n" +
                    "      if (item.type == \"txt\") {\n" +
                    "        titleTxts.push(item.text);\n" +
                    "      }\n" +
                    "\n" +
                    "      if (item.type == \"img\") {\n" +
                    "        titleImgs.push(item.text);\n" +
                    "\n" +
                    "      }\n" +
                    "    });\n" +
                    "\n" +
                    "");
            //沒有圖片的試卷, 不用找圖檔
            if (hasImgFolder.get() == true) {

                writer.append("" +
                        "    let position = 0;\n" +
                        "    titleImgs.forEach((item) => {\n\n" +
                        "    let decode = Utilities.base64Decode(imagesDB[item]);\n" +
                        "        if (position == 0) {\n" +
                        "          //加入題目說明及圖片\n" +
                        "          form.addImageItem()\n" +
                        "            .setTitle(\"Q.\" + titleTxts[0])\n" +
                        "            .setImage(Utilities.newBlob(decode, MimeType.JPEG, item));\n" +
                        "        } else {\n" +
                        "          //第二張開始不用加入題目說明\n" +
                        "          form.addImageItem()\n" +
                        "            .setTitle(\"\")\n" +
                        "            .setImage(Utilities.newBlob(decode, MimeType.JPEG, item));\n" +
                        "        }\n" +
                        "\n\n" +
                        "      position += 1;\n" +
                        "    })\n" +
                        "");

            }
            writer.append("" +
                    "\n" +
                    "    //取得正確答案\n" +
                    "    let ans = question.ans[0].text;\n" +
                    "\n" +
                    "    //加入選項\n" +
                    "    let item = form.addMultipleChoiceItem();\n" +
                    "\n" +
                    "    //如果沒有圖片,題目說明要加在MultipleChoiceItem()\n" +
                    "    if (titleImgs.length == 0) {\n" +
                    "      item.setTitle(\"Q.\" + titleTxts[0]);\n" +
                    "    } else {\n" +
                    "   item.setTitle(\"Ans.\");\n" +
                    "}\n" +

                    "\n" +
                    "    let choices = [];\n" +
                    "    question.choices.forEach((option) => {\n" +
                    "\n" +
                    "      let choiceTitle = option.text;\n" +
                    "      let value = (option.index + 1 == ans) ? true : false\n" +
                    "      choices.push(item.createChoice(choiceTitle, value));\n" +
                    "\n" +
                    "    })\n" +
                    "    item.setPoints(scores);\n" +
                    "    item.setChoices(choices);\n" +
                    "    item.setRequired(isAllRequired);\n" +
                    "\n" +
                    "  })\n\n");
            writer.append("Logger.log(\"表單成功建立：\"+ form.getEditUrl());");


            writer.append("}");
            writer.flush();
            writer.close();


        }

    }


}

//            if (hasImgFolder.get() == true) {
//
//                writer.append(String.format("let folders = DriveApp.getFoldersByName(imgFolder);\n"));
//                writer.append(
//                        "  let srcFolder;\n" +
//                                "  if (folders.hasNext) {\n" +
//                                "    srcFolder = folders.next();\n" +
//                                "  }");
//
//                writer.append("\n");
//                writer.append("  let getImg = function (img, srcDir) {\n" +
//                        "    let picDB = srcDir.getFiles();\n" +
//                        "    while (picDB.hasNext()) {\n" +
//                        "      let target = picDB.next();\n" +
//                        "\n" +
//                        "      if (img == target) {\n" +
//                        "\n" +
//                        "        return target;\n" +
//                        "      }\n" +
//                        "    }\n" +
//                        "\n" +
//                        "  }");
//
//            }
